/**
 * OtLoXNX
 */

package tcurve;

import java.awt.*;
import java.text.DecimalFormat;

abstract class GraphCanvas extends Canvas {
    public int swidth, sheight, gwidth, gheight;
    public double xmin, xmax, ymin, ymax, xscale, yscale;
    private int margin_x, margin_y, length_lbl, length_sc;
    private double ax, ay, bx, by;
    private String label_x, label_y;
    private Image buff;
    private Graphics buff_g;
    private DecimalFormat df_x, df_y;

    GraphCanvas() {
    }

    //W̐ݒ
    void setAxis(double xmin, double xmax, double ymin, double ymax, double xscale, double yscale) {
        this.xmin = xmin;
        this.xmax = xmax;
        this.ymin = ymin;
        this.ymax = ymax;
        this.xscale = xscale;
        this.yscale = yscale;
    }

    //x̐ݒ
    void setLabel(String label_x, String label_y, DecimalFormat df_x, DecimalFormat df_y) {
        this.label_x = label_x;
        this.label_y = label_y;
        this.df_x = df_x;
        this.df_y = df_y;
    }

    //}[W̐ݒ
    void setMargin(int margin_x, int margin_y, int length_lbl, int length_sc) {
        this.margin_x = margin_x;
        this.margin_y = margin_y;
        this.length_lbl = length_lbl;
        this.length_sc = length_sc;
    }

    //Ot̕`
    abstract void draw(Graphics g);

    //W̕`
    void drawAxis() {
        for(double x=xmin; x<xmax; x+=xscale) {
            buff_g.setColor(Color.lightGray);
            buff_g.drawLine(transX(x), transY(ymin), transX(x), transY(ymax));
            buff_g.setColor(Color.gray);
            buff_g.drawString(df_x.format(x), transX(x), transY(ymin) + (12 + margin_y));
        }
        for(double y=ymin; y<ymax; y+=yscale) {
            buff_g.setColor(Color.lightGray);
            buff_g.drawLine(transX(xmin), transY(y), transX(xmax), transY(y));
            buff_g.setColor(Color.gray);
            buff_g.drawString(df_y.format(y), transX(xmin) - length_sc, transY(y));
        }
        buff_g.setColor(Color.black);
        buff_g.drawLine(transX(xmin), transY(ymin), transX(xmax), transY(ymin));
        buff_g.drawLine(transX(xmin)-1, transY(ymin)+1, transX(xmax), transY(ymin)+1);
        buff_g.drawLine(transX(xmin), transY(ymin), transX(xmin), transY(ymax));
        buff_g.drawLine(transX(xmin)-1, transY(ymin)+1, transX(xmin)-1, transY(ymax));
        buff_g.drawString(label_x, (gwidth - length_lbl)/2 + (length_lbl + length_sc + 2*margin_x), sheight - margin_y);
        buff_g.drawString(label_y, margin_x, (gheight + 12)/2 + margin_y);
    }

    //Ȑi܂j̕`
    void drawCurve(double[] x, double[] y, Color color) {
        buff_g.setColor(color);
        for(int i=0; i<x.length-1; i++) {
            if(x[i]<xmin || x[i]>xmax) continue;
            buff_g.drawLine(transX(x[i]), transY(y[i]), transX(x[i+1]), transY(y[i+1]));
        }
    }

    //Wϊp[^̐ݒ
    void init() {
        bx = gwidth * (0 - xmin) / (xmax - xmin) + (length_lbl + length_sc + 2 * margin_x);
        ax = gwidth * (1 - xmin) / (xmax - xmin) + (length_lbl + length_sc + 2 * margin_x) - bx;
        by = gheight * (ymax - 0) / (ymax - ymin) + margin_y;
        ay = gheight * (ymax - 1) / (ymax - ymin) + margin_y - by;
    }

    //Ot̍WLoX̍Wɕϊ
    int transX(double x) {
        return (int)(ax * x + bx + 0.5);
    }
    int transY(double y) {
        return (int)(ay * y + by + 0.5);
    }

    public void paint(Graphics g) {
        if(buff == null) {
            swidth = getSize().width;
            sheight = getSize().height;
            gwidth = swidth - (length_lbl + length_sc + 3*margin_x);
            gheight = sheight - (2*12 + 4*margin_y);
            buff = createImage(swidth, sheight);
            buff_g = buff.getGraphics();
        }

        buff_g.setColor(Color.white);
        buff_g.fillRect(0, 0, swidth, sheight);

        init();
        draw(buff_g);

        g.drawImage(buff, 0, 0, this);
    }

    public void update(Graphics g) {
        paint(g);
    }
}