/**
 * pHOtNX
 */

package tcurve;

import java.awt.*;
import java.awt.event.*;
import java.text.DecimalFormat;

class PHGraph extends GraphCanvas implements MouseListener, MouseMotionListener {
    double[] Ka = new double[4];
    double[] Kb = new double[4];
    double Kw, Ca, Cb;
    boolean order, gradient;

    private double[][] ph;
    private int mx, my;

    PHGraph() {
        setAxis(0.0, 3.5, 0.0, 14.5, 0.5, 2.0);
        setLabel("p", "pH", new DecimalFormat("#0.0##"), new DecimalFormat("#0"));
        setMargin(5, 5, 20, 20);

        Kw = 1.0E-14;
        order = true;
        gradient = true;

        addMouseListener(this);
        addMouseMotionListener(this);
    }

    void draw(Graphics g) {
        if(ph == null) ph = new double[3][gwidth];
        calc();
        drawAxis();
        drawCurve(ph[0], ph[1], Color.red);
        if(gradient) drawCurve(ph[0], ph[2], Color.blue);

        if(mx < transX(xmax) && mx > transX(xmin) && my > transY(ymax) && my < transY(ymin)) {
            g.setColor(Color.green);
            g.drawLine(mx, transY(ymin), mx, transY(ymax));
            g.setColor(Color.black);
            g.drawString("( " + new DecimalFormat("#0.000").format(ph[0][mx - transX(xmin)]) + ", " +
                new DecimalFormat("#0.00").format(ph[1][mx - transX(xmin)]) + " )", mx + 10, transY(ymax) + 20);
        }
    }

    private void calc() {
        double dx = (xmax - xmin) / gwidth;
        double x = xmin;

        ph[0][0] = x;
        ph[1][0] = ph(x);
        for(int i=1; i<gwidth; i++) {
            x += dx;
            ph[0][i] = x;
            ph[1][i] = ph(x);
            ph[2][i] = (ph[1][i] - ph[1][i-1])/dx * 0.1;
            if(!order) ph[2][i] *= -1.0;
        }
    }

    void changeScale(double x) {
        xmax = x;
        if(x > 8.0) xscale = 2.0;
        else if(x > 4.0) xscale = 1.0;
        else if(x > 2.0) xscale = 0.5;
        else if(x > 1.0) xscale = 0.25;
        else if(x > 0.5) xscale = 0.125;
        repaint();
    }

    //f[^o
    String getCSVData() {
        StringBuffer sb = new StringBuffer();
        sb.append("# p, pH, dpH/dp\n");
        for(int i=0; i<gwidth; i+=2) {
            sb.append(new DecimalFormat("#0.0000000").format(ph[0][i]));
            sb.append(",");
            sb.append(new DecimalFormat("#0.00000").format(ph[1][i]));
            sb.append(",");
            if(order) sb.append(new DecimalFormat("#0.00000").format(10 * ph[2][i]));
            else sb.append(new DecimalFormat("#0.00000").format(-10 * ph[2][i]));
            sb.append("\n");
        }
        return sb.toString();
    }

    //񕪖@ɂpȞvZ
    private double ph(double p) {
        double ca, cb, H, H1 = 0.0, H2 = 1.0;

        //cs = Cs * Vs / (Vs + Vt)
        //ct = Ct * Vt / (Vs + Vt) = cs * p
        //p = (Vt * Ct) / (Vs * Cs)
        //C : dݔZx  c : ̔Zx

        if(order) {
            ca = Ca / (1 + Ca/Cb * p);
            cb = ca * p;
        } else {
            cb = Cb / (1 + Cb/Ca * p);
            ca = cb * p;
        }

        do {
            H = (H1 + H2)/2;
            if(func(H, ca, cb) < 0) H1 = H;
            else H2 = H;
        } while((H2 / H1 - 1) > 1.0E-4);

        return -Math.log((H1 + H2)/2) / Math.log(10);
    }

    private double func(double H, double ca, double cb) {
        double OH = Kw / H;
        return H - OH
            - (Ka[0]*H*H*H + 2*Ka[0]*Ka[1]*H*H + 3*Ka[0]*Ka[1]*Ka[2]*H + 4*Ka[0]*Ka[1]*Ka[2]*Ka[3])
            / (H*H*H*H + H*H*H*Ka[0] + H*H*Ka[0]*Ka[1] + H*Ka[0]*Ka[1]*Ka[2] + Ka[0]*Ka[1]*Ka[2]*Ka[3]) * ca
            + (Kb[0]*OH*OH*OH + 2*Kb[0]*Kb[1]*OH*OH + 3*Kb[0]*Kb[1]*Kb[2]*OH + 4*Kb[0]*Kb[1]*Kb[2]*Kb[3])
            / (OH*OH*OH*OH + OH*OH*OH*Kb[0] + OH*OH*Kb[0]*Kb[1] + OH*Kb[0]*Kb[1]*Kb[2] + Kb[0]*Kb[1]*Kb[2]*Kb[3]) * cb;
    }

    public void mouseClicked(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
        mx = 0;
        my = 0;
        repaint();
    }
    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }

    public void mouseDragged(MouseEvent e) {
    }
    public void mouseMoved(MouseEvent e) {
        mx = e.getX();
        my = e.getY();
        repaint();
    }
}